import { run, utils, procOf, DefaultLogger, Process } from 'mangojuice-core';

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var get = function get(object, property, receiver) {
  if (object === null) object = Function.prototype;
  var desc = Object.getOwnPropertyDescriptor(object, property);

  if (desc === undefined) {
    var parent = Object.getPrototypeOf(object);

    if (parent === null) {
      return undefined;
    } else {
      return get(parent, property, receiver);
    }
  } else if ("value" in desc) {
    return desc.value;
  } else {
    var getter = desc.get;

    if (getter === undefined) {
      return undefined;
    }

    return getter.call(receiver);
  }
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var TrackableProcess = function (_Process) {
  inherits(TrackableProcess, _Process);

  function TrackableProcess() {
    classCallCheck(this, TrackableProcess);
    return possibleConstructorReturn(this, (TrackableProcess.__proto__ || Object.getPrototypeOf(TrackableProcess)).apply(this, arguments));
  }

  createClass(TrackableProcess, [{
    key: 'exec',
    value: function exec() {
      var _babelHelpers$get;

      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      (_babelHelpers$get = get(TrackableProcess.prototype.__proto__ || Object.getPrototypeOf(TrackableProcess.prototype), 'exec', this)).call.apply(_babelHelpers$get, [this].concat(args));
      return this.finished();
    }
  }, {
    key: 'run',
    value: function run$$1() {
      var _babelHelpers$get2;

      for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      (_babelHelpers$get2 = get(TrackableProcess.prototype.__proto__ || Object.getPrototypeOf(TrackableProcess.prototype), 'run', this)).call.apply(_babelHelpers$get2, [this].concat(args));
      return this.finished();
    }
  }, {
    key: 'update',
    value: function update() {
      var _babelHelpers$get3;

      for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
        args[_key3] = arguments[_key3];
      }

      (_babelHelpers$get3 = get(TrackableProcess.prototype.__proto__ || Object.getPrototypeOf(TrackableProcess.prototype), 'update', this)).call.apply(_babelHelpers$get3, [this].concat(args));
      return this.finished();
    }
  }]);
  return TrackableProcess;
}(Process);

function runWithTracking() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      expectErrors = _ref.expectErrors,
      app = _ref.app;

  var commands = [];
  var commandNames = [];
  var execOrder = [];
  var errors = [];

  var TrackerLogger = function (_DefaultLogger) {
    inherits(TrackerLogger, _DefaultLogger);

    function TrackerLogger() {
      classCallCheck(this, TrackerLogger);
      return possibleConstructorReturn(this, (TrackerLogger.__proto__ || Object.getPrototypeOf(TrackerLogger)).apply(this, arguments));
    }

    createClass(TrackerLogger, [{
      key: 'onCatchError',
      value: function onCatchError(e) {
        if (!expectErrors) {
          console.error(e);
          throw e;
        }
        errors.push(e);
      }
    }, {
      key: 'onEndExec',
      value: function onEndExec(proc, cmd) {
        execOrder.push(cmd);
      }
    }, {
      key: 'onStartExec',
      value: function onStartExec(proc, cmd) {
        commands.push(cmd);
        commandNames.push(cmd);
      }
    }]);
    return TrackerLogger;
  }(DefaultLogger);

  try {
    var logger = new TrackerLogger();
    var runOptions = {
      Process: TrackableProcess,
      logger: logger
    };
    var model = run(app, runOptions);
    var proc = procOf(model);

    var result = {
      commandNames: commandNames,
      execOrder: execOrder,
      commands: commands,
      errors: errors,
      app: { model: model, proc: proc }
    };

    var promise = proc.finished().then(function () {
      return result;
    });
    utils.extend(promise, result);
    return promise;
  } catch (e) {
    return Promise.reject(e);
  }
}

export { runWithTracking };
